@@ -7,7 +7,7 @@ use App::Alice::Stream;
 use App::Alice::CommandDispatch;
 use MIME::Base64;
 use JSON;
-use Moose;
+use Any::Moose;
 
 has 'app' => (
   is  => 'ro',
@@ -21,18 +21,16 @@ has 'httpd' => (
 );
 
 has 'streams' => (
-  traits => ['Array'],
   is  => 'rw',
   auto_deref => 1,
   isa => 'ArrayRef[App::Alice::Stream]',
   default => sub {[]},
-  handles => {
-    add_stream   => 'push',
-    no_streams   => 'is_empty',
-    stream_count => 'count',
-  }
 );
 
+sub add_stream {push @{shift->streams}, @_}
+sub no_streams {@{$_[0]->streams} == 0}
+sub stream_count {scalar @{$_[0]->streams}}
+
 has 'config' => (
   is => 'ro',
   isa => 'App::Alice::Config',
@@ -47,6 +45,7 @@ has 'ping_timer' => (
 sub BUILD {
   my $self = shift;
   my $httpd = AnyEvent::HTTPD->new(
+    host => $self->config->http_address,
     port => $self->config->http_port,
   );
   $httpd->reg_cb(
@@ -60,7 +59,10 @@ sub BUILD {
     '/say'          => sub{$self->handle_message(@_)},
     '/static'       => sub{$self->handle_static(@_)},
     '/get'          => sub{$self->image_proxy(@_)},
+    '/logs'         => sub{$self->send_logs(@_)},
+    '/search'       => sub{$self->send_search(@_)},
     'client_disconnected' => sub{$self->purge_disconnects(@_)},
+    request         => sub{$self->check_authentication(@_)},
   );
   $self->httpd($httpd);
   $self->ping;
@@ -99,12 +101,10 @@ sub broadcast {
 
 sub check_authentication {
   my ($self, $httpd, $req) = @_;
-  unless ($self->config->auth
+  return unless ($self->config->auth
       and ref $self->config->auth eq 'HASH'
       and $self->config->auth->{username}
-      and $self->config->auth->{password}) {
-    $req->respond([200,'ok']) 
-  }
+      and $self->config->auth->{password});
 
   if (my $auth  = $req->headers->{authorization}) {
     $auth =~ s/^Basic //;
@@ -112,13 +112,13 @@ sub check_authentication {
     my ($user,$password)  = split(/:/, $auth);
     if ($self->config->auth->{username} eq $user &&
         $self->config->auth->{password} eq $password) {
-      $req->respond([200,'ok']);
       return;
     }
     else {
       $self->log_debug("auth failed");
     }
   }
+  $httpd->stop_request;
   $req->respond([401, 'unauthorized', {'WWW-Authenticate' => 'Basic realm="Alice"'}]);
 }
 
@@ -129,8 +129,8 @@ sub setup_stream {
   $self->add_stream(
     App::Alice::Stream->new(
       queue   => [
-        map({$_->nicks_action} $self->app->windows),
         $self->app->buffered_messages($msgid),
+        map({$_->nicks_action} $self->app->windows),
       ],
       request => $req,
     )
@@ -192,11 +192,27 @@ sub handle_static {
 
 sub send_index {
   my ($self, $httpd, $req) = @_;
-  my $channels = [];
   my $output = $self->app->render('index');
   $req->respond([200, 'ok', {'Content-Type' => 'text/html; charset=utf-8'}, $output]);
 }
 
+sub send_logs {
+  my ($self, $httpd, $req) = @_;
+  $self->app->logger->refresh_channels(sub {
+    my $output = $self->app->render('logs', $self->app->logger);
+    $req->respond([200, 'ok', {'Content-Type' => 'text/html; charset=utf-8'}, $output]);
+  });
+}
+
+sub send_search {
+  my ($self, $httpd, $req) = @_;
+  my $results = $self->app->logger->search($req->vars, sub {
+    my $rows = shift;
+    my $content = $self->app->render('results', @$rows);
+    $req->respond([200, 'ok', {'Content-Type' => 'text/html; charset=utf-8'},
+      $content]);
+  });
+}
 
 sub send_config {
   my ($self, $httpd, $req) = @_;
