@@ -4,7 +4,7 @@ use Encode;
 use AnyEvent;
 use AnyEvent::IRC::Client;
 use Digest::MD5 qw/md5_hex/;
-use Moose;
+use Any::Moose;
 
 has 'cl' => (
   is      => 'rw',
@@ -49,16 +49,14 @@ has 'reconnect_timer' => (
 );
 
 has 'reconnect_count' => (
-  traits => ['Counter'],
   is  => 'rw',
   isa => 'Int',
   default   => 0,
-  handles   => {
-    increase_reconnect_count => 'inc',
-    reset_reconnect_count => 'reset',
-  },
 );
 
+sub increase_reconnect_count {$_[0]->reconnect_count($_[0]->reconnect_count + 1)}
+sub reset_reconnect_count {$_[0]->reconnect_count(0)}
+
 has [qw/is_connected disabled removed/] => (
   is  => 'rw',
   isa => 'Bool',
@@ -66,20 +64,18 @@ has [qw/is_connected disabled removed/] => (
 );
 
 has nicks => (
-  traits    => ['Hash'],
   is        => 'rw',
   isa       => 'HashRef[HashRef|Undef]',
   default   => sub {{}},
-  handles   => {
-    remove_nick   => 'delete',
-    includes_nick => 'exists',
-    get_nick_info => 'get',
-    all_nicks     => 'keys',
-    all_nick_info => 'values',
-    set_nick_info => 'set',
-  }
 );
 
+sub remove_nick {delete $_[0]->nicks->{$_[1]}}
+sub includes_nick {exists $_[0]->nicks->{$_[1]}}
+sub get_nick_info {$_[0]->nicks->{$_[1]}}
+sub all_nicks {keys %{$_[0]->nicks}}
+sub all_nick_info {values %{$_[0]->nicks}}
+sub set_nick_info {$_[0]->nicks->{$_[1]} = $_[2]}
+
 sub BUILD {
   my $self = shift;
   $self->cl->enable_ssl(1) if $self->config->{ssl};
@@ -97,12 +93,15 @@ sub BUILD {
     privatemsg     => sub{$self->privatemsg(@_)},
     connect        => sub{$self->connected(@_)},
     disconnect     => sub{$self->disconnected(@_)},
+    dcc_request    => sub{$self->dcc_request(@_)},
+    dcc_connected  => sub{$self->dcc_connected(@_)},
     irc_352        => sub{$self->irc_352(@_)}, # WHO info
     irc_366        => sub{$self->irc_366(@_)}, # end of NAMES
     irc_372        => sub{$self->log_info($_[1]->{params}[1], 1)}, # MOTD info
     irc_377        => sub{$self->log_info($_[1]->{params}[1], 1)}, # MOTD info
     irc_378        => sub{$self->log_info($_[1]->{params}[1], 1)}, # MOTD info
   );
+  $self->cl->ctcp_auto_reply ('VERSION', ['VERSION', "alice $App::Alice::VERSION"]);
   $self->connect unless $self->disabled;
 }
 
@@ -118,6 +117,12 @@ sub window {
            $title, $self);
 }
 
+sub find_window {
+  my ($self, $title) = @_;
+  $title = decode("utf8", $title, Encode::FB_QUIET);
+  return $self->app->find_window($title, $self);
+}
+
 sub nick {
   my $self = shift;
   my $nick = $self->cl->nick;
@@ -206,12 +211,12 @@ sub registered {
   });
   for (@{$self->config->{on_connect}}) {
     push @log, $self->log_info("sending $_");
-    $self->cl->send_raw($_);
+    $self->cl->send_raw(split /\s+/);
   }
 
   for (@{$self->config->{channels}}) {
     push @log, $self->log_info("joining $_");
-    $self->cl->send_srv("JOIN", $_);
+    $self->cl->send_srv("JOIN", split /\s+/);
   }
   $self->app->send(\@log);
 };
@@ -248,97 +253,106 @@ sub remove {
 
 sub publicmsg {
   my ($self, $cl, $channel, $msg) = @_;
-  my $nick = (split '!', $msg->{prefix})[0];
-  my $text = $msg->{params}[1];
-  my $window = $self->window($channel);
-  $self->app->send([$window->format_message($nick, $text)]);
+  if (my $window = $self->find_window($channel)) {
+    my $nick = (split '!', $msg->{prefix})[0];
+    return if $self->app->is_ignore($nick);
+    my $text = $msg->{params}[1];
+    $self->app->logger->log_message(time, $nick, $channel, $text);
+    $self->app->send([$window->format_message($nick, $text)]); 
+  }
 }
 
 sub privatemsg {
   my ($self, $cl, $nick, $msg) = @_;
+  my $text = $msg->{params}[1];
   if ($msg->{command} eq "PRIVMSG") {
     my $from = (split /!/, $msg->{prefix})[0];
+    return if $self->app->is_ignore($from);
     my $window = $self->window($from);
-    $self->app->send([$window->format_message($from, $msg->{params}[1])]); 
+    $self->app->logger->log_message(time, $from, $from, $text);
+    $self->app->send([$window->format_message($from, $text)]); 
   }
   elsif ($msg->{command} eq "NOTICE") {
-    $self->app->send([$self->log_info($msg->{params}[1])]);
+    $self->app->send([$self->log_info($text)]);
   }
 }
 
 sub ctcp_action {
   my ($self, $cl, $nick, $channel, $msg, $type) = @_;
-  my $window = $self->window($channel);
-  $self->app->send([$window->format_message($nick, "• $msg")]);
+  return if $self->app->is_ignore($nick);
+  if (my $window = $self->find_window($channel)) {
+    my $text = "• $msg";
+    $self->app->logger->log_message(time, $nick, $channel, $text);
+    $self->app->send([$window->format_message($nick, $text)]);
+  }
 }
 
 sub nick_change {
   my ($self, $cl, $old_nick, $new_nick, $is_self) = @_;
   $self->rename_nick($old_nick, $new_nick);
   $self->app->send([
-    map {$_->format_event("nick", $old_nick, $new_nick)}
-        $self->nick_windows($old_nick)
+    map  {$_->format_event("nick", $old_nick, $new_nick)}
+    grep {$_} $self->nick_windows($new_nick)
   ]);
 }
 
 sub _join {
   my ($self, $cl, $nick, $channel, $is_self) = @_;
-  my $window = $self->window($channel);
-  if ($is_self) {
-    $self->cl->send_srv("WHO $channel");
+  if (!$self->includes_nick($nick)) {
+    $self->add_nick($nick, {nick => $nick, channels => {$channel => ''}}); 
   }
   else {
+    $self->get_nick_info($nick)->{channels}{$channel} = '';
+  }
+  if ($is_self) {
+    $self->app->create_window($channel, $self);
+    $self->cl->send_srv("WHO $channel");
+  }
+  elsif (my $window = $self->find_window($channel)) {
     $self->cl->send_srv("WHO $nick");
-    if (!$self->includes_nick($nick)) {
-      $self->add_nick($nick, {nick => $nick, channels => {$channel => ''}}); 
-    }
-    else {
-      $self->get_nick_info($nick)->{channels}{$channel} = '';
-    }
     $self->app->send([$window->format_event("joined", $nick)]);
   }
 }
 
 sub channel_add {
   my ($self, $cl, $msg, $channel, @nicks) = @_;
-  my $window = $self->window($channel);
-  for (@nicks) {
-    if (!$self->includes_nick($_)) {
-      $self->add_nick($_, {nick => $_, channels => {$channel => ''}}); 
-    }
-    else {
-      $self->get_nick_info($_)->{channels}{$channel} = '';
-    }
+  if (my $window = $self->find_window($channel)) {
+    for (@nicks) {
+      if (!$self->includes_nick($_)) {
+        $self->add_nick($_, {nick => $_, channels => {$channel => ''}}); 
+      }
+      else {
+        $self->get_nick_info($_)->{channels}{$channel} = '';
+      }
+    } 
   }
 }
 
 sub part {
   my ($self, $cl, $nick, $channel, $is_self, $msg) = @_;
-  if ($is_self) {
-    my $window = $self->app->find_window($channel, $self);
-    if ($window) {
-      $self->app->send([$self->log_info("leaving $channel")]);
-      $self->app->close_window($window);
-    }
+  if ($is_self and my $window = $self->find_window($channel)) {
+    $self->app->send([$self->log_info("leaving $channel")]);
+    $self->app->close_window($window);
   }
 }
 
 sub channel_remove {
   my ($self, $cl, $msg, $channel, @nicks) = @_;
-  return unless @nicks;
-  return if grep {$_ eq $self->nick} @nicks;
-  my $window = $self->window($channel);
-  $self->remove_nicks(@nicks);
-  $self->app->send([
-    map {$window->format_event("left", $_, $msg->{params}[0])} @nicks
-  ]);
+  return if !@nicks or grep {$_ eq $self->nick} @nicks;
+  if (my $window = $self->find_window($channel)) {
+    $self->remove_nicks(@nicks);
+    $self->app->send([
+      map {$window->format_event("left", $_, $msg->{params}[0])} @nicks
+    ]);
+  }
 }
 
 sub channel_topic {
   my ($self, $cl, $channel, $topic, $nick) = @_;
-  my $window = $self->window($channel);
-  $window->topic({string => $topic, author => $nick, time => time});
-  $self->app->send([$window->format_event("topic", $nick, $topic)]);
+  if (my $window = $self->find_window($channel)) {
+    $window->topic({string => $topic, author => $nick, time => time});
+    $self->app->send([$window->format_event("topic", $nick, $topic)]);
+  }
 }
 
 sub channel_nicks {
@@ -355,7 +369,7 @@ sub nick_channels {
 sub nick_windows {
   my ($self, $nick) = @_;
   if ($self->nick_channels($nick)) {
-    return map {$self->window($_)} $self->nick_channels($nick);
+    return map {$self->find_window($_)} $self->nick_channels($nick);
   }
   return;
 }
@@ -378,17 +392,15 @@ sub irc_352 {
       %{$prev_info->{channels}},
       %{$info->{channels}},
     }
-  }  
+  }
   $self->set_nick_info($nick, $info);
 }
 
 sub irc_366 {
   my ($self, $cl, $msg) = @_;
-  my $window = $self->window($msg->{params}[1]);
-  $self->app->send([
-    $window->join_action,
-    $window->nicks_action,
-  ]);
+  if (my $window = $self->find_window($msg->{params}[1])) {
+    $self->app->send([$window->join_action]);
+  }
 }
 
 sub rename_nick {
@@ -415,7 +427,7 @@ sub nick_avatar {
   my ($self, $nick) = @_;
   my $info = $self->get_nick_info($nick);
   if ($info and $info->{real}) {
-    if ($info->{real} =~ /.+@.+/) {
+    if ($info->{real} =~ /[^<\S]+@[^>\S]+/) {
       return "//www.gravatar.com/avatar/"
            . md5_hex($info->{real}) . "?s=32&amp;r=x";
     }
